/*
 * test-dup.c -- test buf_dup
 *
 * Copyright (c) 2019-2021 David Demelier <markand@malikania.fr>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#define GREATEST_USE_ABBREVS 0
#include <greatest.h>

#include "buf.h"

GREATEST_TEST
dup_set(void)
{
	struct buf b = {0};
	struct buf d = {0};

	GREATEST_ASSERT_EQ(0, buf_puts(&b, "0123456789"));
	GREATEST_ASSERT_EQ(0, buf_dup(&d, &b));
	GREATEST_ASSERT(b.data != d.data);
	GREATEST_ASSERT_STR_EQ(d.data, "0123456789");
	GREATEST_ASSERT_EQ(d.length, 10U);
	GREATEST_ASSERT_EQ(d.capacity, 10U);
	buf_finish(&d);

	GREATEST_PASS();
}

GREATEST_TEST
dup_empty(void)
{
	struct buf b = {0};
	struct buf d = {0};

	GREATEST_ASSERT_EQ(0, buf_dup(&d, &b));
	GREATEST_ASSERT(!d.data);
	GREATEST_ASSERT_EQ(d.length, 0U);
	GREATEST_ASSERT_EQ(d.capacity, 0U);
	buf_finish(&d);

	GREATEST_PASS();
}

GREATEST_MAIN_DEFS();

int
main(int argc, char **argv)
{
	GREATEST_MAIN_BEGIN();
	GREATEST_RUN_TEST(dup_set);
	GREATEST_RUN_TEST(dup_empty);
	GREATEST_MAIN_END();
}
