/*
 * test-erase.c -- test buf_erase
 *
 * Copyright (c) 2019-2021 David Demelier <markand@malikania.fr>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#define GREATEST_USE_ABBREVS 0
#include <greatest.h>

#include "buf.h"

GREATEST_TEST
erase(void)
{
	struct buf b = {0};

	buf_puts(&b, "0123456789");

	/* Only remove portion from 1 to 3, capacity must not change. */
	buf_erase(&b, 1, 3);
	GREATEST_ASSERT_STR_EQ(b.data, "0456789");
	GREATEST_ASSERT_EQ(b.length, 7U);
	GREATEST_ASSERT_EQ(b.capacity, 10U);

	/* Remove from 4 to the end. */
	buf_erase(&b, 1, -1);
	GREATEST_ASSERT_STR_EQ(b.data, "0");
	GREATEST_ASSERT_EQ(b.length, 1U);
	GREATEST_ASSERT_EQ(b.capacity, 10U);
	buf_finish(&b);

	GREATEST_PASS();
}

GREATEST_MAIN_DEFS();

int
main(int argc, char **argv)
{
	GREATEST_MAIN_BEGIN();
	GREATEST_RUN_TEST(erase);
	GREATEST_MAIN_END();
}
