/*
 * test-reserve.c -- test buf_reserve
 *
 * Copyright (c) 2019-2021 David Demelier <markand@malikania.fr>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#define GREATEST_USE_ABBREVS 0
#include <greatest.h>

#include "buf.h"

GREATEST_TEST
reserve(void)
{
	struct buf b = {0};

	/* Reserve allocates but does not change length. */
	GREATEST_ASSERT_EQ(0, buf_reserve(&b, 100U));
	GREATEST_ASSERT(b.data);
	GREATEST_ASSERT_EQ(b.length, 0U);
	GREATEST_ASSERT_EQ(b.capacity, 100U);

	/*
	 * Now we have a capacity of 100, if I request 100 again it should not
	 * change capacity.
	 */
	GREATEST_ASSERT_EQ(0, buf_reserve(&b, 100U));
	GREATEST_ASSERT(b.data);
	GREATEST_ASSERT_EQ(b.length, 0U);
	GREATEST_ASSERT_EQ(b.capacity, 100U);

	/* But if I reserve more, it should. */
	GREATEST_ASSERT_EQ(0, buf_reserve(&b, 200U));
	GREATEST_ASSERT(b.data);
	GREATEST_ASSERT_EQ(b.length, 0U);
	GREATEST_ASSERT_EQ(b.capacity, 200U);

	buf_finish(&b);

	GREATEST_PASS();
}

GREATEST_MAIN_DEFS();

int
main(int argc, char **argv)
{
	GREATEST_MAIN_BEGIN();
	GREATEST_RUN_TEST(reserve);
	GREATEST_MAIN_END();
}
