/*
 * test-resize.c -- test buf_resize
 *
 * Copyright (c) 2019-2021 David Demelier <markand@malikania.fr>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#define GREATEST_USE_ABBREVS 0
#include <greatest.h>

#include "buf.h"

GREATEST_TEST
resize_bigger(void)
{
	struct buf b = {0};

	/* Resize actually change the length with the specified character. */
	GREATEST_ASSERT_EQ(0, buf_resize(&b, 10, 'a'));
	GREATEST_ASSERT_STR_EQ(b.data, "aaaaaaaaaa");
	GREATEST_ASSERT_EQ(b.length, 10U);
	GREATEST_ASSERT_EQ(b.capacity, 10U);

	/* Add some more letters. */
	GREATEST_ASSERT_EQ(0, buf_resize(&b, b.length + 5, 'b'));
	GREATEST_ASSERT_STR_EQ(b.data, "aaaaaaaaaabbbbb");
	GREATEST_ASSERT_EQ(b.length, 15U);
	GREATEST_ASSERT_EQ(b.capacity, 20U);

	buf_finish(&b);

	GREATEST_PASS();
}

GREATEST_TEST
resize_smaller(void)
{
	struct buf b = {0};

	GREATEST_ASSERT_EQ(0, buf_resize(&b, 10, 'a'));
	GREATEST_ASSERT_STR_EQ(b.data, "aaaaaaaaaa");
	GREATEST_ASSERT_EQ(b.length, 10U);
	GREATEST_ASSERT_EQ(b.capacity, 10U);

	/* Resize smaller, the letter 'b' should not even appear. */
	GREATEST_ASSERT_EQ(0, buf_resize(&b, 5, 'b'));
	GREATEST_ASSERT_STR_EQ(b.data, "aaaaa");
	GREATEST_ASSERT_EQ(b.length, 5U);
	GREATEST_ASSERT_EQ(b.capacity, 10U);

	buf_finish(&b);

	GREATEST_PASS();
}

GREATEST_MAIN_DEFS();

int
main(int argc, char **argv)
{
	GREATEST_MAIN_BEGIN();
	GREATEST_RUN_TEST(resize_bigger);
	GREATEST_RUN_TEST(resize_smaller);
	GREATEST_MAIN_END();
}
