/*
 * test-shrink.c -- test buf_shrink
 *
 * Copyright (c) 2019-2021 David Demelier <markand@malikania.fr>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#define GREATEST_USE_ABBREVS 0
#include <greatest.h>

#include "buf.h"

GREATEST_TEST
shrink_set(void)
{
	struct buf b = {0};

	/* Reserve 10 but write 5 character, shrink should set a capacity of 5. */
	GREATEST_ASSERT_EQ(0, buf_reserve(&b, 10U));
	GREATEST_ASSERT_EQ(0, buf_puts(&b, "abcde"));
	GREATEST_ASSERT_EQ(0, buf_shrink(&b));
	GREATEST_ASSERT_STR_EQ(b.data, "abcde");
	GREATEST_ASSERT_EQ(b.length, 5U);
	GREATEST_ASSERT_EQ(b.capacity, 5U);
	buf_finish(&b);

	GREATEST_PASS();
}

GREATEST_TEST
shrink_empty(void)
{
	struct buf b = {0};

	/*
	 * Reserve 10, the data should be allocated but since we have a null
	 * length it should be completely free'd.
	 */
	GREATEST_ASSERT_EQ(0, buf_reserve(&b, 10U));
	GREATEST_ASSERT_EQ(0, buf_shrink(&b));
	GREATEST_ASSERT(!b.data);
	GREATEST_ASSERT_EQ(b.length, 0U);
	GREATEST_ASSERT_EQ(b.capacity, 0U);
	buf_finish(&b);

	GREATEST_PASS();
}

GREATEST_TEST
shrink_null(void)
{
	struct buf b = {0};

	GREATEST_ASSERT_EQ(0, buf_shrink(&b));
	GREATEST_ASSERT(!b.data);
	GREATEST_ASSERT_EQ(b.length, 0U);
	GREATEST_ASSERT_EQ(b.capacity, 0U);
	buf_finish(&b);

	GREATEST_PASS();
}

GREATEST_MAIN_DEFS();

int
main(int argc, char **argv)
{
	GREATEST_MAIN_BEGIN();
	GREATEST_RUN_TEST(shrink_set);
	GREATEST_RUN_TEST(shrink_empty);
	GREATEST_RUN_TEST(shrink_null);
	GREATEST_MAIN_END();
}
