/*
 * test-sub.c -- test buf_sub
 *
 * Copyright (c) 2019-2021 David Demelier <markand@malikania.fr>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#define GREATEST_USE_ABBREVS 0
#include <greatest.h>

#include "buf.h"

GREATEST_TEST
sub(void)
{
	struct buf b = {0};
	struct buf s1, s2;

	buf_puts(&b, "0123456789");

	/* Copy only the portion from 1 to 3. */
	GREATEST_ASSERT_EQ(0, buf_sub(&s1, &b, 1, 3));
	GREATEST_ASSERT_STR_EQ(s1.data, "123");
	GREATEST_ASSERT_EQ(s1.length, 3U);
	GREATEST_ASSERT_EQ(s1.capacity, 3U);
	buf_finish(&s1);

	/* Copy whole string starting from 4. */
	GREATEST_ASSERT_EQ(0, buf_sub(&s2, &b, 4, -1));
	GREATEST_ASSERT_STR_EQ(s2.data, "456789");
	GREATEST_ASSERT_EQ(s2.length, 6U);
	GREATEST_ASSERT_EQ(s2.capacity, 6U);
	buf_finish(&s2);
	buf_finish(&b);

	GREATEST_PASS();
}

GREATEST_MAIN_DEFS();

int
main(int argc, char **argv)
{
	GREATEST_MAIN_BEGIN();
	GREATEST_RUN_TEST(sub);
	GREATEST_MAIN_END();
}
